{% extends 'admin/change_list.html' %}
{% load i18n %}

{% block object-tools-items %}
	{{ block.super }}
	
	<li>
		<a href="boilerplate/">
			{% blocktrans %}Download boilerplate{% endblocktrans %}
		</a>
	</li>
	
	<li>
		<a href="upload/{% if is_popup %}?_popup=1{% endif %}" class="addlink">
			{% blocktrans with cl.opts.verbose_name as name %}Upload {{ name }}{% endblocktrans %}
		</a>
	</li>
{% endblock %}